package simple;

import java.io.*;

/**
 *	Formal Semantics of Programming Languages
 *	SS 99
 *	1. Exercise
 *
 *	Class represents domain Store.
 *	Domain Store maps locations to storable-values.
 *
 *	@author Jrgen Hartl, 9756179 / 881
 *	@version 1.0 (JDK 1.1.7)
 */
public class Store extends Domain
{
	Function map = null;
	Location top = null;
	
	/* creates empty store where values are 0. Function needed by interpreter */
	public static Store empty()
	{
		return new Store((StorableValue)null, Location.first());
	}

  public String contentPrint()
  {
  	StringBuffer text = new StringBuffer("(Store: Top=" + top.toString() + ", Mapping:");
		Location l = Location.first();
    StorableValue sval = (StorableValue)map.access(l);

    while (sval != null)
    {
    	text.append("\n" + l.toString() + "\t--->\t" + sval.toString());
    	l = l.next();
      sval = (StorableValue)map.access(l);
    }
    return text.toString();
  }

	public String toString()
	{
		return "(Store: Top=" + top.toString() + ", Mapping:\n" + map.toString();
	}

	public StoreaccessValue access(Location l)
	{
		Tr tr = l.lessThan(top);
		
		if (tr.isTrue() != null)
		{
			return StoreaccessValue.inStorableValue((StorableValue)map.access(l));
		}
		else
		if (tr.isFalse() != null)
		{
			return StoreaccessValue.inErrvalue();
		}
		return null;
	}
	
	public Poststore update(Location l, StorableValue sval)
	{
		Tr tr = l.lessThan(top);
		
		if (tr.isTrue() != null)
		{
			return Poststore.inOk(new Store(map.update(l, sval), top));
		}
		else
		if (tr.isFalse() != null)
		{
			return Poststore.inErr(this);
		}
		return null;
	}
	
	public Location markLocn()
	{
		return top;
	}
	
	public AllocateLocnResult allocateLocn()
	{
		return new AllocateLocnResult(top, Poststore.inOk(new Store(map, top.next())));
	}
	
	public Poststore deallocateLocn(Location l)
	{
		Tr tr = l.lessEqualThan(top);
		
		if (tr.isTrue() != null)
		{
			return Poststore.inOk(new Store(map, l));
		}
		else
		if (tr.isFalse() != null)
		{
			return Poststore.inErr(this);
		}
		return null;
	}
	
	public GetstorageResult getStorage(Num n, Array arr)
	{
		Tr tr = n.greaterThan(arr.upper);
		
		if (tr.isTrue() != null)
		{
			return new GetstorageResult(DenotableValue.inArray(arr), Poststore.inOk(this));
		}
		else
		if (tr.isFalse() != null)
		{
			AllocateLocnResult res = allocateLocn();
			Store s = null;
			
			if ((s = res.p.isOk()) != null)
			{
				return s.getStorage(n.plus(Num.ONE()), arr.augment(n, ReferenceableValue.inLocation(res.l)));
			}
			else
			if ((s = res.p.isErr()) != null)
			{
				return new GetstorageResult(DenotableValue.inErrvalue(), Poststore.inErr(s));
			}
		}
		return null;
	}

	protected Store(StorableValue defaultValue, Location top)
	{
		map = new Function(defaultValue);
		this.top = top;
	}
	
	protected Store(Function map, Location top)
	{
		this.map = map;
		this.top = top;
	}
}